' File name: Verify.inc
' Project:  Register Dialog

'Include WINAPI.INC for Windows API
'$include 'winapi.inc'

' FW_ Flag combinations used in the code generated by the Recorder
Const FW_BASE_FLAGS      =  FW_FOCUS or FW_IGNOREFILE or FW_ERROR
Const FINDWINDOWFLAGS    =  FW_BASE_FLAGS
Const FINDWINDOWFLAGS_IF =  FW_FOCUS or FW_IGNOREFILE_RIGHT or FW_IGNOREFILE_LEFT or FW_ERROR
Const FIND_AND_MAX       =  FW_IGNOREFILE or FW_ERROR or FW_MAXIMIZE
Const FIND_AND_MIN       =  FW_IGNOREFILE or FW_ERROR or FW_MINIMIZE

Const FW_BASE_FLAGS_NA   =  FW_PART or FW_IGNOREFILE or FW_ERROR
Const FINDWINDOWFLAGS_NA =  FW_BASE_FLAGS_NA
Const FIND_AND_MAX_NA    =  FW_BASE_FLAGS_NA or FW_MAXIMIZE
Const FIND_AND_MIN_NA    =  FW_BASE_FLAGS_NA or FW_MINIMIZE

Const EPSILON            = 0.000001

'Verification constants
Const V_WINDOW          = 0
Const V_BUTTON          = 1
Const V_CHECK           = 2
Const V_EDIT            = 3
Const V_LIST            = 4
Const V_COMBO           = 5
Const V_SCROLL          = 6
Const V_LISTVIEW        = 7
Const V_HEADER          = 8
Const V_TREEVIEW        = 9
Const V_TOOLBAR         = 10
Const V_TAB             = 11
Const V_SLIDER          = 12
Const V_SPIN            = 13
Const V_PROGRESS        = 14
Const V_STATUS          = 15
Const V_TIPS            = 16
Const V_OPTION          = 17
Const V_MENU            = 18
Const V_STATIC          = 19
Const V_DESKTOP			= 20

Const VS_NOTCHECKED     = 0
Const VS_CHECKED        = 1
Const VS_GRAYED         = 2
Const VS_NOTGRAYED      = 3

'Variable declarations used in the script
GLOBAL CurrentWindow       As LONG
GLOBAL ChildCurrentWindow  As LONG
GLOBAL Timeout             AS LONG

'Verification Function declarations
'------------------------------------------------------------------------------
Declare Sub CheckResolution         (x&, y&)

Declare Sub RecVerifyExistence      (szControl$, szItem$, dwType&, bExists&)
Declare Sub RecVerifyEnabled        (szControl$, szItem$, dwType&, bEnabled&)
Declare Sub RecVerifyFocus          (szControl$, dwType&, bFocus&)
Declare Sub RecVerifyState          (szControl$, uState&, dwType&)
Declare Sub RecVerifyCharPos        (szControl$, uPos&, dwType&)
Declare Sub RecVerifyLine           (szControl$, uLine&, dwType&)
Declare Sub RecVerifyText           (szControl$, szItem$, nItem&, dwType&)
Declare Sub RecVerifySelected       (szControl$, szItem$, dwType&, bSelected&)
Declare Sub RecVerifyFirstVisible   (szControl$, uTop&)
Declare Sub RecVerifyCount          (szControl$, uCount&, dwType&, bSelected&)
Declare Sub RecVerifyMax            (szControl$, uMax&, dwType&)
Declare Sub RecVerifyMin            (szControl$, uMin&, dwType&)
Declare Sub RecVerifyPos            (szControl$, uPos&, dwType&)
Declare Sub RecVerifyViewMode       (szControl$, uMode&)
Declare Sub RecVerifyExpanded       (szControl$, szItem$, bExpanded&)
Declare Sub RecVerifyIndex          (szControl$, szItem$, nIndex&, dwType&)
Declare Sub RecVerifyCmdId          (szControl$, szItem$, uID&)
Declare Sub RecVerifyPage           (szControl$, uPage&, dwType&)
Declare Sub RecVerifyLocation       (szControl$, szClass$, nLeft&, nTop&, nRight&, nBottom&, dwType&)
Declare Sub RecVerifyCompare		(szFile$, szImage$, szCaption$)
Declare Sub RecVerifyOleProperty    (szControl$, szClassname$, szProp$, vPropVal as VARIANT)
Declare Sub RecVerifyArrayProperty  (szControl$, szClassname$, szProp$, vPropVal as VARIANT, vIndex as array of variant)

'Support routines
'-----------------------------------------------------------------------------
Declare Function RecGetMenuItem(szMenuPath$) as STRING
Declare Function PullDownPopups(szMenuPath$) as INTEGER
Declare Function GetDesktopItemIndex(szItem$) as LONG

Timeout = 5

' Sub: CheckResolution

' Desc: This sub will compare the width and height specified by 'x' and 'y'
'       to the real width and height of the screen.  If they
'       do not match, a FAIL is raised.

Sub CheckResolution(x&, y&)

    if (GetSystemMetrics (SM_CXSCREEN) <> x) or (GetSystemMetrics (SM_CYSCREEN) <> y) then
        FAIL "This script was recorded under a different resolution than the current one."
    endif

End Sub

' Sub: RecVerifyExistence
'
' Desc: This sub will verify that the window or control specified
'       by 'szControl$' exists.  The type of control is denoted
'       by the 'dwType' parameter.  If the window does not exist,
'       a FAIL is raised.
Sub RecVerifyExistence(szControl$, szItem$, dwType&, bExists&)

    select case dwType
        case V_WINDOW
			if bExists& then
				if WFndWndWaitC(szControl$, szItem$, FW_FULL, Timeout) = 0 then
					LogData( "The window """ + szControl$ + """ does not exist.")
				endif
			else
				if NOT WFndWndWait(szControl$, FW_NOEXIST or FW_FULL, Timeout) then
					LogData( "The window """ + szControl$ + """ exists.")
				endif
			endif
		
        case V_BUTTON
            if NOT WButtonExists(szControl$) then
                LogData( "The button """ + szControl$ + """ does not exist.")
            endif

        case V_CHECK
            if NOT WCheckExists(szControl$) then
                LogData("The check box control """ + szControl$ + """ does not exist.")
            endif

        case V_EDIT
            if NOT WEditExists(szControl$) then
                LogData( "The edit control """ + szControl$ + """ does not exist.")
            endif

        case V_LIST
            if ISNULL(szItem$) then
                if NOT WListExists(szControl$) then
                    LogData( "The list box """ + szControl$ + """ does not exist.")
                endif
            else
                if WListItemExists(szControl$, szItem$) <> bExists& then
					if bExists& then
						LogData( "The  list box item """ + szItem$ + """ does not exist.")
					else
						LogData( "The  list box item """ + szItem$ + """ exists.")
					endif
                endif
            endif

        case V_COMBO
            if ISNULL(szItem$) then
                if NOT WComboExists(szControl$) then
                    LogData( "The combo box """ + szControl$ + """ does not exist.")
                endif
            else
                if WComboItemExists(szControl$, szItem$) <> bExists& then
					if bExists& then
						LogData( "The combo box item """ + szItem$ + """ does not exist.")
					else
						LogData( "The combo box item """ + szItem$ + """ exists.")
					endif
                endif
            endif

        case V_SCROLL
            if NOT WScrollExists(szControl$) then
                LogData( "The scroll bar """ + szControl$ + """ does not exist.")
            endif

        case V_LISTVIEW
            if ISNULL(szItem$) then
                if NOT WViewExists(szControl$) then
                    LogData( "The listview """ + szControl$ + """ does not exist.")
                endif
            else
                if WViewItemExists(szControl$, szItem$) <> bExists& then
					if bExists& then
						LogData( "The list view item """ + szItem$ + """ does not exist.")
					else
						LogData( "The list view item """ + szItem$ + """ exists.")
					endif
                endif
            endif

        case V_HEADER
            if ISNULL(szItem$) then
                if NOT WHeaderExists(szControl$) then
					LogData( "The header """ + szControl$ + """ does not exist.")
                endif
            else
                if WHeaderItemExists(szControl$, szItem$) <> bExists& then
                    if bExists& then
					    LogData( "The header item """ + szItem$ + """ does not exist.")
                    else
                        LogData( "The header item """ + szItem$ + """ exists.")
                    endif
                endif
            endif

        case V_TREEVIEW
            if ISNULL(szItem$) then
                if NOT WTreeExists(szControl$) then
                    LogData( "The tree view """ + szControl$ + """ does not exist.")
                endif
            else
                if WTreeItemExists(szControl$, szItem$) <> bExists& then
					if bExists& then
						LogData( "The tree view item """ + szItem$ + """ does not exist.")
					else
						LogData( "The tree view item """ + szItem$ + """ exists.")
					endif
                endif
            endif

        case V_TOOLBAR
            if ISNULL(szItem$) then
                if NOT WToolbarExists(szControl$) then
                    LogData( "The toolbar """ + szControl$ + """ does not exist.")
                endif
            else
                if WToolbarButtonExists(szControl$, szItem$) <> bExists& then
					if bExists& then
						LogData( "The toolbar button """ + szItem$ + """ does not exist.")
					else
						LogData( "The toolbar button """ + szItem$ + """ exists.")
					endif
                endif
            endif

        case V_TAB
            if ISNULL(szItem$) then
                if NOT WTabExists(szControl$) then
                    LogData( "The tab control """ + szControl$ + """ does not exist.")
                endif
            else
                if WTabItemExists(szControl$, szItem$) <> bExists& then
					if bExists& then
						LogData( "The tab item """ + szItem$ + """ does not exist.")
					else
						LogData( "The tab item """ + szItem$ + """ exists.")
					endif
                endif
            endif

        case V_SLIDER
            if NOT WSliderExists(szControl$) then
                LogData( "The slider """ + szControl$ + """ does not exist.")
            endif

        case V_SPIN
            if NOT WSpinExists(szControl$) then
                LogData( "The spin """ + szControl$ + """ does not exist.")
            endif

        case V_PROGRESS
            if NOT WProgressExists(szControl$) then
                LogData( "The progress """ + szControl$ + """ does not exist.")
            endif

        case V_STATUS
            if NOT WStatusExists(szControl$) then
                LogData( "The status """ + szControl$ + """ does not exist.")
            endif

        case V_TIPS
            if NOT WTipsExists(szControl$) then
                LogData( "The tips """ + szControl$ + """ does not exist.")
            endif

        case V_OPTION
            if NOT WOptionExists(szControl$) then
                LogData( "The option """ + szControl$ + """ does not exist.")
            endif

        case V_MENU
            if ISNULL(szItem$) then
                if GetMenu(WFndWndWait(szControl$, FW_CASE or FW_FULL, Timeout)) = 0 then
                    LogData( "A menu does not exist for window """ + szControl$ + """")
                endif
            else
                if RecPullDownPopups(szItem$) then
			        if WMenuExists(RecGetMenuItem(szItem$)) <> bExists then
						if bExists then
                            WMenuEnd()
							LogData( "The menu item """ + szItem$ + """ does not exist.")
						else
                            WMenuEnd()
							LogData( "The menu item """ + szItem$ + """ exists.")
						endif
                    endif

                    WMenuEnd()
                else
                    LogData( "A menu pop-up did not exist - unable to verify menu item.")
                endif
            endif

        case V_STATIC
            if WStaticExists(szControl$) <> bExists& then
				if bExists& then
					LogData( "The static text """ + szControl$ + """ does not exist.")
				else
					LogData( "The static text """ + szControl$ + """ exists.")
				endif
            endif

		case V_DESKTOP
            if WDesktopItemExists(szItem$) <> bExists& then
				if bExists& then
					LogData( "The desktop item """ + szItem$ + """ does not exist.")
				else
					LogData( "The desktop item """ + szItem$ + """ exists.")
				endif
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

' Sub: RecVerifyEnabled
'
' Desc: This sub will verify that a window or control is either
'       enabled or disabled, depending on the value of 'bEnabled'.
'       A FAIL is raised if the state is not correct.

Sub RecVerifyEnabled(szControl$, szItem$, dwType&, bEnabled&)

    DIM     rc as INTEGER
    DIM     hWnd as LONG

    select case dwType
        case V_WINDOW
            hWnd = WFndWndWait(szControl$, FW_FULL or FW_CASE, TimeOut)

            rc = IsWindowEnabled(hWnd)

            if bEnabled then
                if NOT rc then
                    LogData( "The window """ + szControl$ + """ is not enabled.")
                endif
            else
                if rc then
                    LogData( "The window """ + szControl$ + """ is enabled."	 )
                endif
            endif

        case V_BUTTON
            if WButtonEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The button """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The button """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_CHECK
            if WCheckEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The check box """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The check box """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_EDIT
            if WEditEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The edit box """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The edit box """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_LIST
            if WListEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The list box """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The list box """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_COMBO
            if WComboEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The combo box """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The combo box """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_SCROLL
            if WScrollEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The scroll bar """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The scroll bar """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_LISTVIEW
            if WViewEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The list view """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The list view """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_HEADER
            if WHeaderEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The header """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The header """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_TREEVIEW
            if WTreeEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The tree view """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The tree view """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_TOOLBAR
            if WToolbarButtonEnabled(szControl$, szItem$) <> bEnabled then
                if bEnabled then
                    LogData( "The toolbar button """ + szItem$ + """ is not enabled.")
                else
                    LogData( "The toolbar button """ + szItem$ + """ is enabled.")
                endif
            endif

        case V_SLIDER
            if WSliderEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The slider """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The slider """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_SPIN
            if WSpinEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The spin control """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The spin control """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_OPTION
            if WOptionEnabled(szControl$) <> bEnabled then
                if bEnabled then
                    LogData( "The option button """ + szControl$ + """ is not enabled.")
                else
                    LogData( "The option button """ + szControl$ + """ is enabled.")
                endif
            endif

        case V_MENU
            if RecPullDownPopups(szItem$) then
			    if WMenuEnabled(RecGetMenuItem(szItem$)) <> bEnabled then
                    if bEnabled then
                        WMenuEnd()
                        LogData( "The menu item """ + szItem$ + """ is not enabled.")
                    else
                        WMenuEnd()
                        LogData( "The menu item """ + szItem$ + """ is enabled.")
                    endif
                endif

                WMenuEnd()
            else
                LogData( "A menu pop-up did not exist - unable to verify menu item.")
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

' Sub: RecVerifyFocus
'
' Desc: This sub will verify that a window or control either has the
'       focus or does not have it, depending on the value of 'bHasFocus'.
'       A FAIL is raised if the state is not correct.

Sub RecVerifyFocus(szControl$, dwType&, bHasFocus&)

    DIM     hWnd&
    DIM     szTitle$

    select case dwType
        case V_WINDOW
            hWnd = GetFocus()

            szTitle$ = GetText(hWnd)

            if szTitle$ <> szControl$ then
                if bHasFocus then
                    LogData( "The window """ + szControl$ + """ does not have focus.")
                else
                    LogData( "The window """ + szControl$ + """ has focus.")
                endif
            endif

        case V_BUTTON
            if WButtonFocus(szControl$) <> bHasFocus then
                if bHasFocus then
                    LogData( "The button """ + szControl$ + """ does not have focus.")
                else
                    LogData( "The button """ + szControl$ + """ has focus.")
                endif
            endif

        case V_CHECK
            if WCheckFocus(szControl$) <> bHasFocus then
                if bHasFocus then
                    LogData( "The check box """ + szControl$ + """ does not have focus.")
                else
                    LogData( "The check box """ + szControl$ + """ has focus.")
                endif
            endif

        case V_EDIT
			if WEditFocus(szControl$) <> bHasFocus then
				if bHasFocus then
					LogData( "The edit box """ + szControl$ + """ does not have focus.")
				else
					LogData( "The edit box """ + szControl$ + """ has focus.")
				endif
			endif

        case V_LIST
			if WListFocus(szControl$) <> bHasFocus then
				if bHasFocus then
					LogData( "The list box""" + szControl$ + """ does not have focus.")
				else
					LogData( "The list box """ + szControl$ + """ has focus.")
				endif
			endif

        case V_COMBO
			if WComboFocus(szControl$) <> bHasFocus then
				if bHasFocus then
					LogData( "The combo box """ + szControl$ + """ does not have focus.")
				else
					LogData( "The combo box """ + szControl$ + """ has focus.")
				endif
			endif

        case V_SCROLL
			if WScrollFocus(szControl$) <> bHasFocus then
				if bHasFocus then
					LogData( "The scroll bar """ + szControl$ + """ does not have focus.")
				else
					LogData( "The scroll bar """ + szControl$ + """ has focus.")
				endif
			endif

        case V_LISTVIEW
			if WViewFocus(szControl$) <> bHasFocus then
				if bHasFocus then
					LogData( "The list view """ + szControl$ + """ does not have focus.")
				else
					LogData( "The list view """ + szControl$ + """ has focus.")
				endif
			endif

        case V_TREEVIEW
			if WTreeFocus(szControl$) <> bHasFocus then
				if bHasFocus then
					LogData( "The tree view """ + szControl$ + """ does not have focus.")
				else
					LogData( "The tree view """ + szControl$ + """ has focus.")
				endif
			endif

        case V_TAB
			if WTabFocus(szControl$) <> bHasFocus then
				if bHasFocus then
					LogData( "The tab control """ + szControl$ + """ does not have focus.")
				else
					LogData( "The tab control """ + szControl$ + """ has focus.")
				endif
			endif

        case V_SLIDER
			if WSliderFocus(szControl$) <> bHasFocus then
				if bHasFocus then
					LogData( "The slider """ + szControl$ + """ does not have focus.")
				else
					LogData( "The slider """ + szControl$ + """ has focus.")
				endif
			endif

        case V_SPIN
			if WSpinFocus(szControl$) <> bHasFocus then
				if bHasFocus then
					LogData( "The spin control """ + szControl$ + """ does not have focus.")
				else
					LogData( "The spin control """ + szControl$ + """ has focus.")
				endif
			endif

        case V_OPTION
			if WOptionFocus(szControl$) <> bHasFocus then
				if bHasFocus then
					LogData( "The option button """ + szControl$ + """ does not have focus.")
				else
					LogData( "The option button """ + szControl$ + """ has focus.")
				endif
			endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

' Sub: RecVerifyState
'
' Desc: This sub will verify that the state of the control, denoted
'       by 'szControl$' matches the value specified by the 'uState'
'       parameter.  If there is no match, a FAIL is raised.

Sub RecVerifyState(szControl$, uState&, dwType&)

    select case dwType&
        case V_CHECK
            if WCheckState(szControl$) <> uState& then
                if uState& = VS_CHECKED then
                    LogData( "The check box """ + szControl$ + """ was not checked.")
                elseif uState& = VS_NOTCHECKED then
                    LogData( "The check box """ + szControl$ + """ was checked.")
                elseif uState& = VS_NOTGRAYED then
                    LogData( "The check box """ + szControl$ + """ was grayed.")
                elseif uState& = VS_GRAYED then
                    LogData( "The check box """ + szControl$ + """ was not grayed.")
                else
                    LogData( "Illegal function call.")
                endif
            endif

        case V_MENU
            if RecPullDownPopups(szControl$) then
                select case uState&
                    case VS_CHECKED
			            if NOT WMenuChecked(RecGetMenuItem(szControl$)) then
                            WMenuEnd()
                            LogData( "The menu item """ + RecGetMenuItem(szControl$) + """ is not checked.")
                        endif

                    case VS_NOTCHECKED
			            if WMenuChecked(RecGetMenuItem(szControl$)) then
                            WMenuEnd()
                            LogData( "The menu item """ + RecGetMenuItem(szControl$) + """ is checked.")
                        endif

                    case VS_GRAYED
			            if NOT WMenuGrayed(RecGetMenuItem(szControl$)) then
                            WMenuEnd()
                            LogData( "The menu item """ + RecGetMenuItem(szControl$) + """ is not grayed.")
                        endif

                    case VS_NOTGRAYED
			            if WMenuGrayed(RecGetMenuItem(szControl$)) then
                            WMenuEnd()
                            LogData( "The menu item """ + RecGetMenuItem(szControl$) + """ is grayed.")
                        endif

                    case else
                        WMenuEnd()
                        LogData( "Illegal function call.")
                end select

                WMenuEnd()
            else
                LogData( "A menu pop-up did not exist - unable to verify menu item.")
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

' Sub: RecVerifyCharPos
'
' Desc: This sub will verify that the current character position
'       of the edit box matches the position specified by the
'       'uPos' parameter.  If there is no match, a FAIL is raised.

Sub RecVerifyCharPos(szControl$, uPos&, dwType&)

    select case dwType&
        case V_EDIT
            if WEditChar(szControl$) <> uPos& then
                LogData( "Character position of edit control is incorrect.")
            endif
    end select

End Sub

' Sub: RecVerifyLine
'
' Desc: This sub will verify that the current line of the control
'       is equal to the value in the 'uLine' parameter.  If they
'       are not equal, a FAIL is raised.

Sub RecVerifyLine(szControl$, uLine&, dwType&)

    select case dwType&
        case V_EDIT
            if WEditLine(szControl$) <> uLine& then
                LogData( "The current line of the edit control is incorrect.")
            endif

        case V_SLIDER
            if WSliderLine(szControl$) <> uLine& then
                LogData( "The current line of the slider control is incorrect.")
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

' Sub: RecVerifyText
'
' Desc: This sub will verify that the value of some text in a control
'       is equal to the text specified by the 'szItem$' parameter.
'
'       If 'szItem' is non-NULL, then the text to be verified belongs
'       to an item in the control, and not the control itself.
'
'       A FAIL is raised if the text of 'nItem' doesn't match 'szItem$'.

Sub RecVerifyText(szControl$, szItem$, nItem&, dwType&)

    DIM hWnd as LONG

    select case dwType&
        case V_WINDOW
			if GetText(WGetActWnd(0)) <> szItem$ then
				LogData( "The active window does not have the caption """ + szItem$ + """.")
			endif

        case V_BUTTON
            hWnd = WButtonFind(szControl$)

            if hWnd then
                if GetText(hWnd) <> szItem$ then
                    LogData( "The text of button """ + szControl$ + """ is not """ + szItem$ + """.")
                endif
            else
                LogData( "Unable to find the button with ordinal: " + szControl$)
            endif

        case V_CHECK
            hWnd = WCheckFind(szControl$)

            if hWnd then
                if GetText(hWnd) <> szItem$ then
                    LogData( "The text of check box """ + szControl$ + """ is not """ + szItem$ + """.")
                endif
            else
                LogData( "Unable to find the check box with ordinal: " + szControl$)
            endif

        case V_EDIT
			if nItem = 0 then
	            if EditText(szControl$) <> szItem$ then
				    LogData( "The edit box """ + szControl$ + """ does not contain the text """ + szItem$ + """.")
			    endif
            else
                hWnd = WEditFind(szControl$)

                if hWnd then
                    if LabelText(hWnd) <> szItem$ then
                        LogData( "The label associated with the edit box is not """ + szItem$ + """.")
                    endif
                else
                    LogData( "Unable to find the edit box with ordinal: " + szControl$)
                endif
            endif

        case V_LIST
            if nItem <> 0 then
			    if ListItemText(szControl$, nItem&) <> szItem$ then
				    LogData( "Item "+ str$(nItem&)+" in the list box """ + szControl$ + """ is not """ + szItem$ + """.")
			    endif
            else
                hWnd = WListFind(szControl$)

                if hWnd then
                    if LabelText(hWnd) <> szItem$ then
                        LogData( "The label associated with the list box is not """ + szItem$ + """.")
                    endif
                else
                    LogData( "Unable to find the list box with ordinal: " + szControl$)
                endif
            endif

        case V_COMBO
			if nItem& > 0 then
				if ComboItemText(szControl$, nItem&) <> szItem$ then
					LogData( "Item "+str$(nItem&)+" in the combo box """ + szControl$ + """ is not """ + szItem$ + """.")
				endif
			elseif nItem = 0 then
				if ComboText(szControl$) <> szItem$ then
					LogData( "The combo box """ + szControl$ + """ does not contain the text """ + szItem$ + """.")
				endif
			else
                hWnd = WComboFind(szControl$)

                if hWnd then
                    if LabelText(hWnd) <> szItem$ then
                        LogData( "The label associated with the combo box is not """ + szItem$ + """.")
                    endif
                else
                    LogData( "Unable to find the combo box with ordinal: " + szControl$)
                endif
            endif

        case V_LISTVIEW
            if nItem <> 0 then
			    if ViewItemText(szControl$, "@" + LTRIM$(STR$(nItem&)), NULL) <> szItem$ then
				    LogData( "Item "+str$(nItem&)+" in the listview """ + szControl$ + """ is not """ + szItem$ + """.")
			    endif
            else
                hWnd = WViewFind(szControl$)

                if hWnd then
                    if LabelText(hWnd) <> szItem$ then
                        LogData( "The label associated with the list view is not """ + szItem$ + """.")
                    endif
                else
                    LogData( "Unable to find the list view control with ordinal: " + szControl$)
                endif
            endif

        case V_HEADER
			if HeaderItemText(szControl$, "@" + LTRIM$(STR$(nItem&))) <> szItem$ then
				LogData( "Item "+str$(nItem&)+" in the header control """ + szControl$ + """ is not """ + szItem$ + """.")
			endif

        case V_TREEVIEW
            if nItem <> 0 then
			    if TreeItemText(szControl$, "@" + LTRIM$(STR$(nItem&))) <> szItem$ then
				    LogData( "Item "+str$(nItem&)+" in the tree control """ + szControl$ + """ is not """ + szItem$ + """.")
			    endif
            else
                hWnd = WTreeFind(szControl$)

                if hWnd then
                    if LabelText(hWnd) <> szItem$ then
                        LogData( "The label associated with the tree view is not """ + szItem$ + """.")
                    endif
                else
                    LogData( "Unable to find the tree view control with ordinal: " + szControl$)
                endif
            endif

        case V_TOOLBAR
			if ToolbarButtonText(szControl$, "@" + LTRIM$(STR$(nItem&))) <> szItem$ then
				LogData( "Button "+str$(nItem&)+" in the toolbar """ + szControl$ + """ is not """ + szItem$ + """.")
			endif

        case V_TAB
			if TabItemText(szControl$, "@" + LTRIM$(STR$(nItem&))) <> szItem$ then
				LogData( "Item "+str$(nItem&)+" in the tab control """ + szControl$ + """ is not """ + szItem$ + """.")
			endif

        case V_STATUS
			if StatusItemText(szControl$, "@" + LTRIM$(STR$(nItem&))) <> szItem$ then
				LogData( "Item "+str$(nItem&)+" in the status bar """ + szControl$ + """ is not """ + szItem$ + """.")
			endif

        case V_MENU
            if RecPullDownPopups(szItem$) then
			    if RecGetMenuItem(szItem$) <> MenuText("@" + LTRIM$(STR$(nItem))) then
                    WMenuEnd()
                    LogData( "The text of menu item """+str$(nItem&)+""" is not """ + szItem$ + ".")
                endif

                WMenuEnd()
            else
                LogData( "A menu pop-up did not exist - unable to verify menu item.")
            endif

        case V_STATIC
			if LEFT$(StaticText(szControl$), LEN(szItem$)) <> szItem$ then
				LogData( "The static control """ + szControl$ + """ does not contain the text """ + szItem$ + """.")
			endif

        case V_DESKTOP
			if DesktopItemText("@" + LTRIM$(STR$(nItem&))) <> szItem$ then
				LogData( "Item "+str$(nItem&)+" on the desktop is not """ + szItem$ + """.")
			endif

        case V_SLIDER
            hWnd = WSliderFind(szControl$)

            if hWnd then
                if LabelText(hWnd) <> szItem$ then
                    LogData( "The label associated with the slider control is not """ + szControl$ + """.")
                endif
            else
                LogData( "Unable to find slider control with ordinal: " + szControl$)
            endif

        case V_OPTION
            hWnd = WOptionFind(szControl$)

            if hWnd then
                if GetText(hWnd) <> szItem$ then
                    LogData( "The text of option button """ + szControl$ + """ is not """ + szItem$ + """.")
                endif
            else
                LogData( "Unable to find the option button with ordinal: " + szControl$)
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

' Sub: RecVerifyCount
'
' Desc: This sub will verify that the number of items in a control
'       is equal to the number denoted by the 'uCount&' parameter.
'
'       If bSelected is TRUE, then the number of items verified is
'       restricted to those items which are currently selected.
'
'       If the verification does not succeed, a FAIL is raised.

Sub RecVerifyCount(szControl$, uCount&, dwType&, bSelected&)

    select case dwType&
        case V_LIST
            if bSelected then
                if uCount& <> WListSelCount(szControl$) then
                    LogData( "Number of selected items in the list box """ + szControl$ + """ does not equal "+str$(uCount&)+".")
                endif
            else
                if uCount& <> WListCount(szControl$) then
                    LogData( "Number of items in the list box does not equal "+str$(uCount&)+".")
                endif
            endif

        case V_COMBO
            if uCount& <> WComboCount(szControl$) then
                LogData( "Number of items in the combo box does not equal "+str$(uCount&)+".")
            endif

        case V_LISTVIEW
            if bSelected then
                if uCount& <> WViewSelCount(szControl$) then
                    LogData( "Number of selected items in the list view """ + szControl$ + """ does not equal "+str$(uCount&)+".")
                endif
            else
                if uCount& <> WViewCount(szControl$) then
                    LogData( "Number of items in the list view does not equal "+ str$(uCount&)+".")
                endif
            endif

        case V_HEADER
            if uCount& <> WHeaderCount(szControl$) then
                LogData( "Number of items in the header does not equal "+str$(uCount&)+".")
            endif

        case V_TREEVIEW
            if uCount& <> WTreeCount(szControl$) then
                LogData( "Number of items in the tree view does not equal "+str$(uCount&)+".")
            endif

        case V_TOOLBAR
            if uCount& <> WToolbarCount(szControl$) then
                LogData( "Number of buttons in the toolbar does not equal "+str$(uCount&)+".")
            endif

        case V_TAB
            if uCount& <> WTabCount(szControl$) then
                LogData( "Number of items in the tab does not equal "+str$(uCount&)+".")
            endif

        case V_MENU
            if RecPullDownPopups(szControl$) then
			    if WMenuCount() <> uCount& then
                    WMenuEnd()
                    LogData( "Number of items in the menu does not equal "+str$(uCount&)+".")
                endif

                WMenuEnd()
            else
                LogData( "A menu pop-up did not exist - unable to verify menu item count.")
            endif

		case V_DESKTOP
            if bSelected then
                if uCount& <> WDesktopSelCount() then
                    LogData( "Number of selected items on the Desktop does not equal "+str$(uCount&)+".")
                endif
            else
                if uCount& <> WDesktopCount() then
                    LogData( "Number of items on the Desktop does not equal "+ str$(uCount&)+".")
                endif
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

' Sub: RecVerifyFirstVisible
'
' Desc: This sub will verify that the first visible line in a
'       list box matches the value specified by 'uTop&'.  If they
'       do not match, a FAIL is raised.

Sub RecVerifyFirstVisible(szControl$, uTop&)

    if uTop& <> WListTopIndex(szControl$) then
        LogData( "First visible item in the list box is not at index " + LTRIM$(STR$(uTop&)) + ".")
    endif

End Sub

' Sub: RecVerifyMax
'
' Desc: This function will verify that the maximum range of a control
'       is equal to the value specified by 'uMax&'.  If they are not
'       equal, a FAIL is raised.

Sub RecVerifyMax(szControl$, uMax&, dwType&)

    select case dwType&
        case V_SCROLL
            if uMax& <> WScrollMax(szControl$) then
                LogData( "Maximum value of scroll bar control is not " + LTRIM$(STR$(uMax&)) + ".")
            endif

        case V_SLIDER
            if uMax& <> WSliderMax(szControl$) then
                LogData( "Maximum value of slider control is not " + LTRIM$(STR$(uMax&)) + ".")
            endif

        case V_SPIN
            if uMax& <> WSpinMax(szControl$) then
                LogData( "Maximum value of spin control is not " + LTRIM$(STR$(uMax&)) + ".")
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

' Sub: RecVerifyMin
'
' Desc: This function will verify that the minimum range of a control
'       is equal to the value specified by 'uMin&'.  If they are not
'       equal, a FAIL is raised.

Sub RecVerifyMin(szControl$, uMin&, dwType&)

    select case dwType&
        case V_SCROLL
            if uMin& <> WScrollMin(szControl$) then
                LogData( "Minimum value of scroll bar control is not " + LTRIM$(STR$(uMin&)) + ".")
            endif

        case V_SLIDER
            if uMin& <> WSliderMin(szControl$) then
                LogData( "Minimum value of slider control is not " + LTRIM$(STR$(uMin&)) + ".")
            endif

        case V_SPIN
            if uMin& <> WSpinMin(szControl$) then
                LogData( "Minimum value of spin control is not " + LTRIM$(STR$(uMin&)) + ".")
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

' Sub: RecVerifyPos
'
' Desc: This sub will verify that the current position of a control
'       is equal to the value specified by 'uPos&'.  If they are not
'       equal, a FAIL is raised.

Sub RecVerifyPos(szControl$, uPos&, dwType&)

    select case dwType&
        case V_SCROLL
            if uPos& <> WScrollPos(szControl$) then
                LogData( "Current position of scroll bar control is not " + LTRIM$(STR$(uPos&)) + ".")
            endif

        case V_SLIDER
            if uPos& <> WSliderPos(szControl$) then
                LogData( "Current position of slider control is not " + LTRIM$(STR$(uPos&)) + ".")
            endif

        case V_SPIN
            if uPos& <> WSpinPos(szControl$) then
                LogData( "Current position of spin control is not " + LTRIM$(STR$(uPos&)) + ".")
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

' Sub: RecVerifyViewMode
'
' Desc: This sub will verify that the view mode of a list view matches
'       the mode specified by 'uMode&'.  If not, a FAIL is raised.

Sub RecVerifyViewMode(szControl$, uMode&)

    if WViewMode(szControl$) <> uMode& then
        LogData( "View mode of the list view control is not " + LTRIM$(STR$(uMode&)) + ".")
    endif

End Sub

' Sub: RecVerifyExpanded
'
' Desc: This sub will verify that an item in a tree view is either
'       expanded or collapsed, depending on the value of
'       'bExpanded'.  In the event of a failure, a FAIL is raised.

Sub RecVerifyExpanded(szControl$, szItem$, bExpanded&)

    if WTreeItemExpanded(szControl$, szItem$) <> bExpanded then
        if bExpanded then
            LogData( "Tree view item, " + szItem$ + ", is not expanded.")
        else
            LogData( "Tree view item, " + szItem$ + ", is not collapsed.")
        endif
    endif

End Sub

' Sub: RecVerifyCmdId
'
' Desc: This sub will verify that the command ID of a toolbar
'       button matches the value passed in by 'uID&'.  If they
'       do not match, a FAIL is raised.

Sub RecVerifyCmdId(szControl$, szButton$, uID&)

    if uID& <> WToolbarButtonCmdId(szControl$, szButton$) then
        LogData( "Command ID value of button, " + szButton$ + ", does not equal " + LTRIM$(STR$(uID&)) + ".")
    endif

End Sub

' Sub: RecVerifyPage
'
' Desc: This sub will verify the current page of a control to see
'       that it matches the value specified by 'uPage&'.  If not,
'       a FAIL is raised.

Sub RecVerifyPage(szControl$, uPage&, dwType&)

    select case dwType&
        case V_SLIDER
            if WSliderPage(szControl$) <> uPage& then
                LogData( "Page value of slider control does not equal " + LTRIM$(STR$(uPage)) + ".")
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

Sub RecVerifySelected(szControl$, szItem$, dwType&, bSelected&)

    select case dwType&
        case V_EDIT
			if EditSelText(szControl$) <> szItem$ then
				LogData( "The selected text in the edit box """ + szControl$ + """ does not equal """ + szItem$ + """.")
			endif

        case V_COMBO
			if ComboSelText(szControl$) <> szItem$ then
				LogData( "The selected text in the combo box """ + szControl$ + """ does not equal """ + szItem$ + """.")
			endif

        case V_LISTVIEW
			if WViewItemSelected(szControl$, szItem$) <> bSelected then
				if bSelected then
					LogData( "The item """ + szItem$ + """ is not selected in the list view.")
				else
					LogData( "The item """ + szItem$ + """ is selected in the list view.")
				endif
			endif

        case V_TREEVIEW
			if WTreeItemSelected(szControl$, szItem$) <> bSelected then
				if bSelected then
					LogData( "The item """ + szItem$ + """ is not selected in the tree view.")
				else
					LogData( "The item """ + szItem$ + """ is selected in the tree view.")
				endif
			endif

        case V_TOOLBAR
			if WToolbarButtonChecked(szControl$, szItem$) <> bSelected then
				if bSelected then
					LogData( "The button """ + szItem$ + """ is not selected.")
				else
					LogData( "The button """ + szItem$ + """ is selected.")
				endif
			endif

        case V_OPTION
			if WOptionState(szControl$) then
				if NOT bSelected then
					LogData( "The option button """ + szControl$ + """ is selected.")
				endif
			else
				if bSelected then
					LogData( "The option buttion """ + szControl$ + """ is not selected.")
				endif
			endif

        case V_DESKTOP
            if WViewItemSelected("=" + str$(WDesktopFind), szItem$) then
                if NOT bSelected then
                    LogData( "The desktop item """ + szItem$ + """ is selected.")
                endif
            else
                if bSelected then
                    LogData( "The desktop item """ + szItem$ + """ is not selected.")
                endif
            endif

        case V_TAB
            if WTabItemSelected(szControl$, szItem$) then
                if NOT bSelected then
                    LogData( "The tab item """ + szItem$ + """ is selected.")
                endif
            else
                if bSelected then
                    LogData( "The tab item """ + szItem$ + """ is not selected.")
                endif
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

Sub RecVerifyIndex(szControl$, szItem$, nIndex&, dwType&)

    select case dwType&
        case V_LIST
            if WListItemIndex(szControl$, szItem$) <> nIndex& then
                LogData( "The index of item """ + szItem$ + """ does not equal "+str$(nIndex&))
            endif

        case V_LISTVIEW
			if WViewItemIndex(szControl$, szItem$) <> nIndex& then
				LogData( "The index of item """ + szItem$ + """ does not equal "+str$(nIndex&))
			endif

        case V_DESKTOP
            if UCASE$(DesktopItemText("@" + LTRIM$(STR$(nIndex&)))) <> UCASE$(szItem$) then
                LogData( "The index of item """ + szItem$ + """ does not equal "+str$(nIndex&))
            endif

        case V_TREEVIEW
			if WTreeItemIndex(szControl$, szItem$) <> nIndex& then
				LogData( "The index of item """ + szItem$ + """ does not equal "+str$(nIndex&))
			endif

        case V_TOOLBAR
			if WToolbarButtonIndex(szControl$, szItem$) <> nIndex& then
				LogData( "The index of item """ + szItem$ + """ does not equal "+str$(nIndex&))
			endif

        case V_TAB
			if WTabItemIndex(szControl$, szItem$) <> nIndex& then
				LogData( "The index of item """ + szItem$ + """ does not equal "+str$(nIndex&))
			endif

        case V_STATUS
            if WStatusItemIndex(szControl$, szItem$) <> nIndex& then
				LogData( "The index of item """ + szItem$ + """ does not equal "+str$(nIndex&))
            endif

        case V_MENU
            if RecPullDownPopups(szItem$) then
			    if RecGetMenuItem(szItem$) <> MenuText("@" + LTRIM$(STR$(nIndex&))) then
                    WMenuEnd()
                    LogData( "The index of menu item """ + szItem$ + """ is not """+str$(nIndex&)+".")
                endif

                WMenuEnd()
            else
                LogData( "A menu pop-up did not exist - unable to verify menu item.")
            endif

        case else
            LogData( "Improper call to verification function!")
    end select

End Sub

' Function: RecVerifyLocation
'
' Desc:     This function will compare the
'           location of the specified window
'           or control and compare it to the
'           coordinates passed in.

Sub RecVerifyLocation(szControl$, szClass$, nLeft&, nTop&, nRight&, nBottom&, dwType&)

    DIM     hWnd as LONG
    DIM     hWndParent as LONG
    DIM     fChild as LONG
    DIM     rc as RECT
    DIM     pt as POINT

    select case dwType&
        case V_WINDOW
            hWnd = WFndWndWaitC(szControl$, szClass$, FW_CASE, Timeout)

        case V_BUTTON
            hWnd = WButtonFind(szControl$)

        case V_EDIT
            hWnd = WEditFind(szControl$)

        case V_LIST
            hWnd = WListFind(szControl$)

        case V_COMBO
            hWnd = WComboFind(szControl$)

        case V_LISTVIEW
            hWnd = WViewFind(szControl$)

        case V_TREEVIEW
            hWnd = WTreeFind(szControl$)

        case V_HEADER
            hWnd = WHeaderFind(szControl$)

        case V_TAB
            hWnd = WTabFind(szControl$)

        case V_TOOLBAR
            hWnd = WToolbarFind(szControl$)

        case V_SCROLL
            hWnd = WScrollFind(szControl$)

        case V_SLIDER
            hWnd = WSliderFind(szControl$)

        case V_SPIN
            hWnd = WSpinFind(szControl$)

        case V_OPTION
            hWnd = WOptionFind(szControl$)

        case V_PROGRESS
            hWnd = WProgressFind(szControl$)

        case V_CHECK
            hWnd = WCheckFind(szControl$)

        case V_STATUS
            hWnd = WStatusFind(szControl$)

        case V_STATIC
            hWnd = WStaticFind(szControl$)

        case else
            FAIL "Improper call to verification function!"
    end select

    if hWnd then
        fChild = GetWindowLong(hWnd, GWL_STYLE) AND WS_CHILD

        GetWindowRect(hWnd, rc)

        if fChild then
            hWndParent = GetParent(hWnd)

            pt.x = rc.wleft
            pt.y = rc.wtop

            ScreenToClient(hWndParent, pt)

            rc.wleft = pt.x
            rc.wtop  = pt.y

            pt.x = rc.wright - rc.wleft
            pt.y = rc.wbottom - rc.wtop

            ScreenToClient(hWndParent, pt)

            rc.wright  = rc.wleft + pt.x
            rc.wbottom = rc.wtop + pt.y
        endif

        if rc.wleft <> nLeft& then
            LogData( "Top left x coordinate of """ + szControl$ + """ is not "+str$(nLeft&))
        endif

        if rc.wtop <> nTop& then
            LogData( "Top left y coordinate of """ + szControl$ + """ is not "+str$(nLeft&))
        endif

        if rc.wright <> nRight& then
            LogData( "Bottom right x coordinate of """ + szControl$ + """ is not "+str$(nLeft&))
        endif

        if rc.wbottom <> nBottom& then
            LogData( "Bottom right y coordinate of """ + szControl$ + """ is not "+str$(nLeft&))
        endif
    else
        LogData( "Unable to find window: """ + szControl$ + """.")
    endif

End Sub

' Sub:	RecVerifyCompare
'
' Desc:	This function will capture the currently
'		active window and compare it to the image
'		specified by 'szImage$' in the file 'szFile$'.

Sub RecVerifyCompare(szFile$, szImage$, szCaption$)

    DIM     hWnd as LONG

    hWnd = WFndWndWait(szCaption$, FW_FULL or FW_CASE or FW_CHILDOK)

	if ScnCompWindow(szFile, szImage$, hWnd&, FALSE, FALSE) then
		LogData( "The image of the current window is different from the image """ + szImage$ + """.")
	endif

End Sub

' Sub:  RecVerifyOleProperty
'
' Desc: This function will test the specified OLE
'       control property against the value passed
'       in.

Sub RecVerifyOleProperty(szControl$, szClassname$, szProp$, vPropVal as VARIANT)

    DIM     bIsClass as INTEGER
    DIM     v as VARIANT
    DIM     bFail as INTEGER

    if NOT IsNull(szClassName$) then
        OleCtlSetClass(szClassName$)
        bIsClass = TRUE
    endif

    if OleCtlHasProperty(szControl$, szProp$) then
        v = OleCtlGetProperty(szControl$, szProp$)

        if (VarType(v) = 4) OR (VarType(v) = 5) then
            if abs(v - vPropVal) > EPSILON then
                bFail = TRUE
            endif
        else
            if v <> vPropVal then
                bFail = TRUE
            endif
        endif

        if bIsClass then
            OleCtlSetClass("")
        endif

        if bFail then
            LogData(    "The value of property """ + szProp$ + """ is not "+vPropVal + ".")
        endif
    else
        LogData( "The OLE control does not have the property: """ + szProp$ + """")
    endif

End Sub

Sub RecVerifyArrayProperty(szControl$, szClassname$, szProp$, vPropVal as VARIANT, vIndex as ARRAY of VARIANT)

    DIM     x(1 to UBOUND(vIndex)) as INTEGER
    DIM     i as INTEGER
    DIM     v as VARIANT
    DIM     bIsClass as INTEGER
    DIM     bFail as INTEGER

    print UBOUND(vIndex)
    print UBOUND(x)

    for i = 1 to UBOUND(vIndex)
        x(i) = vIndex(i)
    next i

    if NOT IsNull(szClassName$) then
        OleCtlSetClass(szClassName$)
        bIsClass = TRUE
    endif

    if OleCtlHasProperty(szControl$, szProp$) then
        v = OleCtlGetItem(szControl$, szProp$, x)

        if (VarType(v) = 4) OR (VarType(v) = 5) then
            if abs(v - vPropVal) > EPSILON then
                bFail = TRUE
            endif
        else
            if v <> vPropVal then
                bFail = TRUE
            endif
        endif

        if bIsClass then
            OleCtlSetClass("")
        endif

        if bFail then
            LogData(    "The value of property """ + szProp$ + """ is not "+vPropVal + ".")
        endif
    else
        LogData( "The OLE control does not have the property: """ + szProp$ + """")
    endif
End Sub

' Function: RecGetMenuItem
'
' Desc:     This function will return a string that
'           contains the menu item, which is the last
'			thing in the menu path.  Since the user had to
'           select a menu item to get here, we know that
'           szMenuPath contains at least one '\' character.
'

Function RecGetMenuItem(szMenuPath$) as STRING

    DIM     nTail as INTEGER
    DIM     i as INTEGER

    for i = 1 to LEN(szMenuPath$)
		if MID$(szMenuPath$, i, 1) = "\" then
			if MID$(szMenuPath$, i+1, 1) <> "\" AND _
			   MID$(szMenuPath$, i-1, 1) <> "\" then
				nTail = i + 1
			endif
		endif
    next i

    ' nTail should be pointing to the first character
    ' of our menu item.

    RecGetMenuItem = MID$(szMenuPath$, nTail)

End Function

' Function:	PullDownPopups
'
' Desc:		This function will pull down all the
'			pop-up menus in the menu path.
'			Each pop-up will be validated as well.

Function RecPullDownPopups(szMenuPath$) as INTEGER

    DIM     nHead as INTEGER
    DIM     nTail as INTEGER
	DIM		nPopups as INTEGER
    DIM     i as INTEGER
    DIM     c as STRING

	nPopups = 0

	'See how many pop-ups there are.
	'A pop-up is followed by a single "\" character.
	for i = 1 to LEN(szMenuPath$)
		if MID$(szMenuPath$, i, 1) = "\" then
			if MID$(szMenuPath$, i+1, 1) <> "\" AND _
			   MID$(szMenuPath$, i-1, 1) <> "\" then
				nPopups = nPopups + 1
			endif
		endif
	next i

	nTail = 1
    nHead = 1

	'Get each pop-up and, if it exists, pull it down.
	for i = 1 to nPopups
		nHead = nHead + INSTR(MID$(szMenuPath$, nHead), "\") - 1

		if MID$(szMenuPath$, nHead + 1) <> "\" AND _
		   MID$(szMenuPath$, nHead - 1) <> "\" then
			c$ = MID$(szMenuPath$, nTail, nHead - nTail)
		
			if WMenuExists(c$) then
				WMenuSelect(c$)
			else
				RecPullDownPopups = FALSE

				Exit Function
			endif

            nHead = nHead + 1
			nTail = nHead
		endif
	next i

	RecPullDownPopups = TRUE

End Function

' Function: GetDesktopItemIndex
'
' Desc:     This function retrieves the text of each item
'           on the desktop.  If a match is found, the index
'           of that item is returned.
Function GetDesktopItemIndex(szItem$) as LONG

    DIM     nCount&
    DIM     szTemp$
    DIM     i&
    DIM     nIndex&

    nIndex = 0
    nCount = WDesktopCount()

    for i = 1 to nCount
        szTemp$ = DesktopItemText("@" + LTRIM$(STR$(i)))

        if szTemp$ = szItem$ then
            nIndex = i
            exit for
        endif
    next i

    GetDesktopItemIndex = nIndex

End Function

'------------------------------------------------------------------------------
' Support code for the Network Distribution APIs
'
Global TalkHandle AS LONG
TalkHandle = 0
Global TalkInited AS SHORT
TalkInited = FALSE

Declare Sub TalkMsgHandler(msg AS SHORT, Handle AS LONG, Station AS SHORT, DataType AS LONG, pData AS VARIANT)

' Sub:   TalkMsgHandler
'
' Desc:  Is the message handler for the Network Distribution APIs. Currently
'        this does nothing as no special processing is needed to support the
'        synchronization APIs, and so is mostly a template.
'
Sub TalkMsgHandler(msg AS SHORT, Handle AS LONG, Station AS SHORT, DataType AS LONG, pData AS VARIANT)
   Dim DType AS SHORT, i AS SHORT, result AS SHORT
   Dim StationName AS STRING
   Dim DSize&
   Dim Message$

   StationName = TalkGetStationName(Handle, Station)

   Select Case(msg)

      Case TALK_CONNECT
         ' Stations should not get this

      Case TALK_HUNGUP
         ' If the host has hung up

      Case TALK_DATA
         ' If the host has sent data back

         Select Case(DataType)
            Case DT_NO_DATA
               ' Do Nothing
            Case Else
               ' Do Nothing
         End Select

      Case TALK_SYNC
         ' Stations should not get this

      Case Else
         ' Ignore other messages
   End Select

End Sub

' Sub:   DoTalkSync
'
' Desc:  If the talk APIs have not been initialized, then it will try
'        to call the host station.  After that, it will try to do the
'        sync to the host.
'
Sub DoTalkSync(HostName$, SyncID$, Timeout&)
    IF (TalkInited = FALSE) THEN
        TalkHandle = TalkCallHost(HostName$, Varptr(TalkMsgHandler))
    ENDIF

    IF (TalkWaitSync(TalkHandle, SyncID$, Timeout&) = FALSE) THEN
        LogData( "The synchronization event " + SyncID$ + " failed to synchronize.")
    ENDIF

End Sub

