
 /* NOTICE:                                             */
 /* THIS CODE IS NOT SUPPORTED, WARRANTED OR GUARANTEED */
 /* IN ANY WAY BY BROOKTREE CORPORATION.                */
 /* IT IS PROVIDED "AS IS" BECAUSE WE BELIEVE IT MAY    */
 /* BE USEFUL TO YOU.                                   */


/////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 1996 Brooktree Corporation. All Rights Reserved.
//
//  Module:
//
//    Bt848API.h
//
//  Abstract:
//
//    Header file for BT848 DLL
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __BT848API_H
#define __BT848API_H

#include "def__.h"      // define __BT848CLASS & __BT848FUNC

#ifndef BYTE
   typedef unsigned char       BYTE;
#endif

#ifdef __BORLANDC__
   #ifndef CLASSLIB_DEFS_H
   #include <classlib/defs.h>
   #endif
#else
   typedef int bool;
#endif

#ifndef __WINDOWS_H
#include <windows.h>
#endif

#include "gpiotype.h"
#include "viddefs.h"
#include "retcode.h"


//===========================================================================
// BT848 DLL API Header File
//===========================================================================

#ifdef __cplusplus
extern "C"
{
#endif

//---------------------------------------------------------------------------
// I2C DATA/CONTROL REGISTER API
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2CIsInitOK( void )
//  Purpose: Check if I2C is initialized successfully
//  Input:   None
//  Output:  None
//  Return:  true or false
//  Note:    Before calling other I2C functions, application should call
//           this function and check for return value is true to ensure
//           initialization is successful.
/////////////////////////////////////////////////////////////////////////////
bool      __BT848FUNC I2CIsInitOK( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CInitHWMode( long freq )
//  Purpose: Initialize I2C for hardware control of SCL and SDA
//  Input:   long freq - frequency (hz) to run SCL at
//  Output:  None
//  Return:  None
//  Note:    After calling I2CIsInitOK(), application should call this
//           function and check for return value is Success before starting
//           hardware communication.
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CInitHWMode( long freq );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CInitSWMode( long freq )
//  Purpose: Initialize I2C for software control of SCL and SDA
//  Input:   long freq - frequency (hz) to run SCL at
//  Output:  None
//  Return:  None
//  Note:    After calling I2CIsInitOK(), application should call this
//           function and check for return value is Success before starting
//           software communication.
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CInitSWMode( long freq );

/////////////////////////////////////////////////////////////////////////////
//  Method:  void I2CSetFreq( long freq )
//  Purpose: Set frequency for SCL
//  Input:   long freq - frequency (hz) to run SCL at. (137.5khz to 2.0625Mhz)
//             PCI frequency 33Mhz: SCL = (412.50Khz to 33.81Khz)
//                           25Mhz: SCL = (312.50Khz to 25.61Khz)
//  Output:  None
//  Return:  None
/////////////////////////////////////////////////////////////////////////////
void      __BT848FUNC I2CSetFreq( long freq );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2CReadDiv( void )
//  Purpose: Obtain value of programmable divider
//  Input:   None
//  Output:  None
//  Return:  Value of programmable divider
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC I2CReadDiv( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CHWRead( BYTE address, BYTE *value )
//  Purpose: Perform a hardware read from the I2C
//  Input:   int address - address to be read from
//  Output:  int *value  - retrieved value
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CHWRead( BYTE address, BYTE *value );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CHWWrite2( BYTE address, BYTE value1 )
//  Purpose:  Perform a hardware write of two bytes to the I2C
//  Input:   int address - address to be written to
//           int value1  - value of 2nd byte to be written
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CHWWrite2( BYTE address, BYTE value1 );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CHWWrite3( BYTE address, BYTE value1, BYTE value2 )
//  Purpose: Perform a hardware write of three bytes to the I2C
//  Input:   int address - address to be written to
//           int value1  - value of 2nd byte to be written
//           int value2  - value of 3rd byte to be written
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CHWWrite3( BYTE address, BYTE value1, BYTE value2 );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CSWStart( void )
//  Purpose: Generate START condition using software control
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CSWStart( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CSWStop( void )
//  Purpose: Generate STOP condition using software control
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CSWStop( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CSWRead( BYTE * value )
//  Purpose: Read a byte from the slave
//  Input:   None
//  Output:  BYTE * value - byte read from slave
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CSWRead( BYTE * value );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CSWWrite( BYTE value )
//  Purpose: Write a byte to the slave
//  Input:   BYTE value - byte to be written to slave
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CSWWrite( BYTE value );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CSWSendACK( void )
//  Purpose: Send ACK to slave
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CSWSendACK( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CSWSendNACK( void )
//  Purpose: Send NACK to slave
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CSWSendNACK( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CSWSetSCL( Level scl )
//  Purpose: Set software SCL value
//  Input:   Level scl - Hi releases the SCL output; Low forces the SCL output low
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CSWSetSCL( Level scl );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2CSWReadSCL( void )
//  Purpose: Read software SCL value
//  Input:   None
//  Output:  None
//  Return:  SCL value
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC I2CSWReadSCL( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CSWSetSDA( Level sda )
//  Purpose: Set software SDA value
//  Input:   Level sda - Hi releases the SDA output; Low forces the SDA output low
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CSWSetSDA( Level sda );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2CSWReadSDA( void )
//  Purpose: Read software SDA value
//  Input:   None
//  Output:  None
//  Return:  SDA value
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC I2CSWReadSDA( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2CSetSync( State sync )
//  Purpose: Set I2C sync value
//  Input:   sync: On  - allow slave to insert wait states
//                 Off - slave cannot insert wait states
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC I2CSetSync( State sync );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2CReadSync( void )
//  Purpose: Read I2C sync value
//  Input:   None
//  Output:  None
//  Return:  Sync value
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC I2CReadSync( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2CGetLastError( void )
//  Purpose: Obtain last error number
//  Input:   None
//  Output:  None
//  Return:  Last error number
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC I2CGetLastError( void );


//---------------------------------------------------------------------------
// GPIO RELATED REGISTERS API
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool GPIOIsInitOK( void )
//  Purpose: Check if GPIO is initialized successfully
//  Input:   None
//  Output:  None
//  Return:  true or false
//  Note:    Before calling other GPIO functions, application should call
//           this function and check for return value is true to ensure
//           initialization is successful.
/////////////////////////////////////////////////////////////////////////////
bool      __BT848FUNC GPIOIsInitOK( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  void SetGPCLKMODE( State s )
//  Purpose: Set or clear GPCLKMODE
//  Input:   State s - On to set; Off to clear
//  Output:  None
//  Return:  None
/////////////////////////////////////////////////////////////////////////////
void      __BT848FUNC SetGPCLKMODE( State s );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int GetGPCLKMODE( void )
//  Purpose: Get value of GPCLKMODE
//  Input:   None
//  Output:  None
//  Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC GetGPCLKMODE( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  void SetGPIOMODE( GPIOMode mode )
//  Purpose: Set GPIO mode
//  Input:   GPIOMode mode - GPIO_NORMAL, GPIO_SPI_OUTPUT, GPIO_SPI_INPUT,
//                           GPIO_DEBUG_TEST
//  Output:  None
//  Return:  None
/////////////////////////////////////////////////////////////////////////////
void      __BT848FUNC SetGPIOMODE( GPIOMode mode );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int GetGPIOMODE( void )
//  Purpose: Get GPIO mode
//  Input:   None
//  Output:  None
//  Input:   int - GPIO_NORMAL, GPIO_SPI_OUTPUT, GPIO_SPI_INPUT, GPIO_DEBUG_TEST
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC GetGPIOMODE( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  void SetGPWEC( State s )
//  Purpose: Set or clear GPWEC
//  Input:   State s - On to set; Off to clear
//  Output:  None
//  Return:  None
/////////////////////////////////////////////////////////////////////////////
void      __BT848FUNC SetGPWEC( State s );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int GetGPWEC( void )
//  Purpose: Get value of GPWEC
//  Input:   None
//  Output:  None
//  Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC GetGPWEC( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  void SetGPINTI( State s )
//  Purpose: Set or clear GPINTI
//  Input:   State s - On to set; Off to clear
//  Output:  None
//  Return:  None
/////////////////////////////////////////////////////////////////////////////
void      __BT848FUNC SetGPINTI( State s );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int GetGPINTI( void )
//  Purpose: Get value of GPINTI
//  Input:   None
//  Output:  None
//  Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC GetGPINTI( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  void SetGPINTC( State s )
//  Purpose: Set or clear GPINTC
//  Input:   State s - On to set; Off to clear
//  Output:  None
//  Return:  None
/////////////////////////////////////////////////////////////////////////////
void      __BT848FUNC SetGPINTC( State s );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int GetGPINTC( void )
//  Purpose: Get value of GPINTC
//  Input:   None
//  Output:  None
//  Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC GetGPINTC( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode SetGPOEBit( int bit, State s )
//  Purpose: Set or clear a bit in GPOE
//  Input:   int bit - bit to be set or clear
//           State s - On to set; Off to clear
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC SetGPOEBit( int bit, State s );

/////////////////////////////////////////////////////////////////////////////
//  Method:  void SetGPOE( DWORD value )
//  Purpose: Set GPOE value
//  Input:   DWORD value - value to be set to
//  Output:  None
//  Return:  None
/////////////////////////////////////////////////////////////////////////////
void      __BT848FUNC SetGPOE( DWORD value );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int GetGPOEBit( int bit )
//  Purpose: Get value of a bit in GPOE
//  Input:   int bit - bit to get value from
//  Output:  None
//  Return:  int - On (1), Off (0), or -1 for parameter error
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC GetGPOEBit( int bit );

/////////////////////////////////////////////////////////////////////////////
//  Method:  DWORD GetGPOE( void )
//  Purpose: Get value of GPOE
//  Input:   None
//  Output:  None
//  Return:  Content of GPOE (DWORD)
/////////////////////////////////////////////////////////////////////////////
DWORD     __BT848FUNC GetGPOE( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode SetGPIEBit( int bit, State s )
//  Purpose: Set or clear a bit in GPIE
//  Input:   int bit - bit to be set or clear
//           State s - On to set; Off to clear
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC SetGPIEBit( int bit, State s );

/////////////////////////////////////////////////////////////////////////////
//  Method:  void SetGPIE( DWORD value )
//  Purpose: Set GPIE value
//  Input:   DWORD value - value to be set to
//  Output:  None
//  Return:  None
/////////////////////////////////////////////////////////////////////////////
void      __BT848FUNC SetGPIE( DWORD value );

/////////////////////////////////////////////////////////////////////////////
//  Method:  int GetGPIEBit( int bit )
//  Purpose: Get value of a bit in GPIE
//  Input:   int bit - bit to get value from
//  Output:  None
//  Return:  int - On (1), Off (0), or -1 for parameter error
/////////////////////////////////////////////////////////////////////////////
int       __BT848FUNC GetGPIEBit( int bit );

/////////////////////////////////////////////////////////////////////////////
//  Method:  DWORD GetGPIE( void )
//  Purpose: Get value of GPIE
//  Input:   None
//  Output:  None
//  Return:  Content of GPIE (DWORD)
/////////////////////////////////////////////////////////////////////////////
DWORD     __BT848FUNC GetGPIE( void );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode SetGPDATA( GPIOReg * data, int size, int offset )
//  Purpose: Set GPDATA registers contents
//  Input:   GPIOReg * data - ptr to data to be copied from
//           int size       - number of registers to be copied; max is 64
//           int offset     - how many registers to skip (default = 0)
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC SetGPDATA( GPIOReg * data, int size, int offset = 0 );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode GetGPDATA( GPIOReg * data, int size, int offset )
//  Purpose: Get GPDATA registers contents
//  Input:   GPIOReg * data - ptr to data to be copied to
//           int size       - number of registers to be copied; max is 64
//           int offset     - how many registers to skip (default = 0)
//  Output:  None
//  Return:  Success or Fail
//  Note:    This function can only retrieve the last value written to GPDATA
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC GetGPDATA( GPIOReg * data, int size, int offset = 0 );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode SetGPDATABits( int fromBit, int toBit,
//                                    DWORD value, int offset )
//  Purpose: Set GPDATA contents with specified range of bits
//  Input:   int fromBit - starting bit
//           int toBit   - ending bit
//           DWORD value - value to set to
//           int offset  - how many registers to skip (default = 0)
//  Output:  None
//  Return:  Success or Fail
//  Comment: By specifying a range of bits to be set, it allows a portion of
//           the GDATA register to be modified. For example,
//              SetGPDATA( 8, 10, 5, 0 )
//           would set the first GPDATA register's [10:8] to binary value 101
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC SetGPDATABits( int fromBit, int toBit,
                                     DWORD value, int offset = 0 );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode GetGPDATABits( int fromBit, int toBit,
//                                    DWORD * value, int offset )
//  Purpose: Get GPDATA contents with specified range of bits
//  Input:   int fromBit - starting bit
//           int toBit   - ending bit
//           int offset  - how many registers to skip (default = 0)
//  Output:  DWORD * value - value retrieved
//  Return:  Success or Fail
//  Comment: By specifying a range of bits to be set, it allows a portion of
//           the GDATA register to be retrieved. For example,
//              GetGPDATA( 8, 10, &data, 0 )
//           would set *data to the first GPDATA register's [10:8] content
//  Note:    This function can only retrieve the last value written to GPDATA
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC GetGPDATABits( int fromBit, int toBit,
                                     DWORD * value, int offset = 0 );


//---------------------------------------------------------------------------
// DECODER REGISTER API
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// The following table contains the decoder register read/write permissions.
//	A "Write" bit allows both read and write operations whereas a bit with
// "Read Only" allows only read operation.
//
// STATUS Adr 0x00 = 0x03
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Write
// Bit 0  Write
// 
// IFORM Adr 0x04 = 0x7F
// Bit 7  Read Only
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Write
// Bit 2  Write
// Bit 1  Write
// Bit 0  Write
// 
// TDEC Adr 0x08 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// CROP Adr 0x0C = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// VDELAY_LO Adr 0x10 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// VACTIVE_LO Adr 0x14 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// HDELAY_LO Adr 0x18 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// HACTIVE_LO Adr 0x1C = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// HSCALE_HI Adr 0x20 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// HSCALE_LO Adr 0x24 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// BRIGHT Adr 0x28 = 0xFF
// Bit 7  Write
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Write
// Bit 2  Write
// Bit 1  Write
// Bit 0  Write
// 
// MISCCONTROL Adr 0x2C = 0xF7
// Bit 7  Write
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Read Only
// Bit 2  Write
// Bit 1  Write
// Bit 0  Write
// 
// CONTR_LO Adr 0x30 = 0xFF
// Bit 7  Write
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Write
// Bit 2  Write
// Bit 1  Write
// Bit 0  Write
// 
// SAT_U_LO Adr 0x34 = 0xFF
// Bit 7  Write
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Write
// Bit 2  Write
// Bit 1  Write
// Bit 0  Write
// 
// SAT_V_LO Adr 0x38 = 0xFF
// Bit 7  Write
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Write
// Bit 2  Write
// Bit 1  Write
// Bit 0  Write
// 
// HUE Adr 0x3C = 0xFF
// Bit 7  Write
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Write
// Bit 2  Write
// Bit 1  Write
// Bit 0  Write
// 
// SCLOOP Adr 0x40 = 0x78
// Bit 7  Read Only
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Write
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// WC_UP Adr 0x44 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// OFORM Adr 0x48 = 0xE0
// Bit 7  Write
// Bit 6  Write
// Bit 5  Write
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// VSCALE_HI Adr 0x4C = 0xC0
// Bit 7  Write
// Bit 6  Write
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// VSCALE_LO Adr 0x50 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// TEST Adr 0x54 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// VPOLE Adr 0x58 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// IDCODE Adr 0x5C = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ADELAY Adr 0x60 = 0xFF
// Bit 7  Write
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Write
// Bit 2  Write
// Bit 1  Write
// Bit 0  Write
// 
// BDELAY Adr 0x64 = 0xFF
// Bit 7  Write
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Write
// Bit 2  Write
// Bit 1  Write
// Bit 0  Write
// 
// ADC Adr 0x68 = 0x3F
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Write
// Bit 4  Write
// Bit 3  Write
// Bit 2  Write
// Bit 1  Write
// Bit 0  Write
// 
// VTC Adr 0x6C = 0x03
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Write
// Bit 0  Write
// 
// RES_1C Adr 0x70 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// RES_1D Adr 0x74 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// RES_1E Adr 0x78 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// SRESET Adr 0x7C = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ODD_CROP Adr 0x8C = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ODD_VDELAY_LO Adr 0x90 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ODD_VACTIVE_LO Adr 0x94 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ODD_HDELAY_LO Adr 0x98 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ODD_HACTIVE_LO Adr 0x9C = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ODD_HSCALE_HI Adr 0xA0 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ODD_HSCALE_LO Adr 0xA4 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ODD_MISCCONTROL Adr 0xAC = 0xF7
// Bit 7  Write
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Read Only
// Bit 2  Write
// Bit 1  Write
// Bit 0  Write
// 
// ODD_SCLOOP Adr 0xC0 = 0x78
// Bit 7  Read Only
// Bit 6  Write
// Bit 5  Write
// Bit 4  Write
// Bit 3  Write
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ODD_VSCALE_HI Adr 0xCC = 0xC0
// Bit 7  Write
// Bit 6  Write
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ODD_VSCALE_LO Adr 0xD0 = 0x00
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Read Only
// Bit 0  Read Only
// 
// ODD_VTC Adr 0xEC = 0x03
// Bit 7  Read Only
// Bit 6  Read Only
// Bit 5  Read Only
// Bit 4  Read Only
// Bit 3  Read Only
// Bit 2  Read Only
// Bit 1  Write
// Bit 0  Write
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode ReadDecoder( WORD regOffset, BYTE * value )
//  Purpose: Read a decoder register's content
//  Input:   WORD regOffset - offset of the register to retrieve (must be in
//                            DWORD boundary)
//  Output:  BYTE * value   - byte read from decoder
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC ReadDecoder( WORD regOffset, BYTE * value );

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode WriteDecoder( WORD regOffset, BYTE value )
//  Purpose: Modify a decoder register's content
//  Input:   WORD regOffset - offset of the register to modify (must be in
//                            DWORD boundary)
//           BYTE value     - value to set to
//  Output:  None
//  Return:  Success or Fail
//  Note:	 Not all bits are permitted to write. Please refer to the above
//				 table for modifiable bits.
/////////////////////////////////////////////////////////////////////////////
ErrorCode __BT848FUNC WriteDecoder( WORD regOffset, BYTE value );


#ifdef __cplusplus
}
#endif

#endif // __BT848API_H
