
 /* NOTICE:                                             */
 /* THIS CODE IS NOT SUPPORTED, WARRANTED OR GUARANTEED */
 /* IN ANY WAY BY BROOKTREE CORPORATION.                */
 /* IT IS PROVIDED "AS IS" BECAUSE WE BELIEVE IT MAY    */
 /* BE USEFUL TO YOU.                                   */


/////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 1996 Brooktree Corporation
//
//  Module:
//
//    Tuner.cpp
//
//  Abstract:
//
//    Perform channel change function through tuner for the Bt848.
//
/////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <mmsystem.h>
#include "tvctl.h"
#include "tverr.h"
#include "tuner.h"
#include "audio.h"
#include "bt848api.h"
#include "bt848fcn.h"


/////////////////////////////////////////////////////////////////////////////
// External variables
/////////////////////////////////////////////////////////////////////////////
extern HINSTANCE hInst;    // Handle to instance


/////////////////////////////////////////////////////////////////////////////
// External functions
/////////////////////////////////////////////////////////////////////////////
LONG SetAudioSource ( ULONG );
LONG SetAudioMute   ( ULONG );


/////////////////////////////////////////////////////////////////////////////
// Following are the functions communicate with I2C
/////////////////////////////////////////////////////////////////////////////
ErrorCode Bt848I2CInit( void );
ErrorCode Bt848I2CWrite( BYTE I2CAddress, WORD msw, WORD lsw );


/////////////////////////////////////////////////////////////////////////////
// Divider codes for NTSC USA Air Channels
/////////////////////////////////////////////////////////////////////////////
TunerCodes airNtscChannelCode[] = {
   0x0000, 0x0000, 0x0650, 0x06B0, 0x0710, 0x07B0,
   0x0810, 0x0DD0, 0x0E30, 0x0E90, 0x0EF0,
   0x0F50, 0x0FB0, 0x1010, 0x2050, 0x20B0,
   0x2110, 0x2170, 0x21D0, 0x2230, 0x2290,
   0x22F0, 0x2350, 0x23B0, 0x2410, 0x2470,
   0x24D0, 0x2530, 0x2590, 0x25F0, 0x2650,
   0x26B0, 0x2710, 0x2770, 0x27D0, 0x2830,
   0x2890, 0x28F0, 0x2950, 0x29B0, 0x2A10,
   0x2A70, 0x2AD0, 0x2B30, 0x2B90, 0x2BF0,
   0x2C50, 0x2CB0, 0x2D10, 0x2D70, 0x2DD0,
   0x2E30, 0x2E90, 0x2EF0, 0x2F50, 0x2FB0,
   0x3010, 0x3070, 0x30D0, 0x3130, 0x3190,
   0x31F0, 0x3250, 0x32B0, 0x3310, 0x3370,
   0x33D0, 0x3430, 0x3490, 0x34F0
};


/////////////////////////////////////////////////////////////////////////////
// Divider codes for NTSC USA Cable Channels
/////////////////////////////////////////////////////////////////////////////
TunerCodes cableNtscChannelCode[] = {
   0x0000, 0x0770, 0x0650, 0x06B0, 0x0710, 0x07B0,
   0x0810, 0x0DD0, 0x0E30, 0x0E90, 0x0EF0,
   0x0F50, 0x0FB0, 0x1010, 0x0A70, 0x0AD0,
   0x0B30, 0x0B90, 0x0BF0, 0x0C50, 0x0CB0,
   0x0D10, 0x0D70, 0x1070, 0x10D0, 0x1130,
   0x1190, 0x11F0, 0x1250, 0x12B0, 0x1310,
   0x1370, 0x13D0, 0x1430, 0x1490, 0x14F0,
   0x1550, 0x15B0, 0x1610, 0x1670, 0x16D0,
   0x1730, 0x1790, 0x17F0, 0x1850, 0x18B0,
   0x1910, 0x1970, 0x19D0, 0x1B30, 0x1A90,
   0x1AF0, 0x1B50, 0x1BB0, 0x1C10, 0x1C70,
   0x1CD0, 0x1D30, 0x1D90, 0x1DF0, 0x1E50,
   0x1EB0, 0x1F10, 0x1F70, 0x1FD0, 0x2030,
   0x2090, 0x20F0, 0x2150, 0x21B0, 0x2210,
   0x2270, 0x22D0, 0x2330, 0x2390, 0x23F0,
   0x2450, 0x24B0, 0x2510, 0x2570, 0x25D0,
   0x2630, 0x2690, 0x26F0, 0x2750, 0x27B0,
   0x2810, 0x2870, 0x28D0, 0x2930, 0x2990,
   0x29F0, 0x2A50, 0x2AB0, 0x2B10, 0x0890,
   0x08F0, 0x0950, 0x09B0, 0x0A10, 0x2B70,
   0x2BD0, 0x2C30, 0x2C90, 0x2CF0, 0x2D50,
   0x2DB0, 0x2E10, 0x2E70, 0x2ED0, 0x2F30,
   0x2F90, 0x2FF0, 0x3050, 0x30B0, 0x3110,
   0x3170, 0x31D0, 0x3230, 0x3290, 0x32F0,
   0x3350, 0x33B0, 0x3410, 0x3470, 0x34D0
};


/////////////////////////////////////////////////////////////////////////////
// Channel Map of frequency bands for NTSC USA air channels
/////////////////////////////////////////////////////////////////////////////
TunerBandStruct airNtscBands[] = {
    {  2,  6, bandVHF_LOW  },
    {  7, 13, bandVHF_HIGH },
    { 14, 69, bandUHF      },
    {  0,  0, 0            } // end marker
};

/////////////////////////////////////////////////////////////////////////////
// Channel Map of frequency bands for NTSC USA cable channels
/////////////////////////////////////////////////////////////////////////////
TunerBandStruct cableNtscBands[] = {
    {   1,   6, bandVHF_LOW  },
    {   7,  13, bandVHF_HIGH },
    {  14,  20, bandVHF_LOW  },
    {  21,  62, bandVHF_HIGH },
    {  63,  94, bandUHF      },
    {  95,  99, bandVHF_LOW  },
    { 100, 125, bandUHF      },
    {   0,   0, 0            }   // end marker
};


/////////////////////////////////////////////////////////////////////////////
ControlWord controlPhilips[] =
{
   0x8E05, // vhf low
   0x8E09, // vhf high
   0x8E0C  // uhf
};

/////////////////////////////////////////////////////////////////////////////
ControlWord controlTemic[] =
{
   0x8E02, // vhf low
   0x8E04, // vhf high
   0x8E01  // uhf
};


//===========================================================================
// Tuner Class
//===========================================================================


/////////////////////////////////////////////////////////////////////////////
// Constructor
//
// Method:  Tuner::Tuner( int brand, int format, BYTE I2CAddress)
// Purpose: Constructor for Tuner class
// Input:   int brand       - brand of tuner(Philips or Temic)
//          int format      - video format (NTSC or PAL)
//          BYTE I2CAddress - I2C address of the tuner
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
Tuner::Tuner( int brand, int format, BYTE I2CAddress)
{
   // there ptrs will pt to the respective look up table contains
   // all the divider and control values for all channels and modes
   m_ptrTunerCodeArray	= NULL;
   m_ptrTunerBandStructArray = NULL;
   m_ptrControlWordArray = NULL;

   m_channel = 0;
   m_format = format;
   m_I2CAddress = I2CAddress;

   // set the brand of tuner
   if ( brand == brandPhilips )
      m_ptrControlWordArray = controlPhilips;
   else if ( brand == brandTemic )
      m_ptrControlWordArray = controlTemic;
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
Tuner::~Tuner()
{
}


/////////////////////////////////////////////////////////////////////////////
// Method:  LONG Tuner::SetChannel( int channel, int mode )
// Purpose: Set channel
// Input:   int channel - channel to set to
//          int mode    - channel mode (air or cable)
// Output:  None
// Return:  TV control error value
/////////////////////////////////////////////////////////////////////////////
LONG Tuner::SetChannel( int channel, int mode )
{
   // to determine the tuner channel codes we need to know the following:
   // 1- the channel #
   // 2- the video format (ntsc or pal)
   // 3- the mode (cable or air)
   // 4- the band setting for the tuner

   // range checking for channel number
   if ( ( channel < 0 ) || ( channel > MaxNumberOfChannel( mode ) ) )
      return ( (LONG)TV_ERR_BAD_VALUE );

   // range checking for channel mode
   if ( ( mode != modeTV ) && ( mode != modeCable ) )
      return ( (LONG)TV_ERR_BAD_VALUE );

   // notify Bt848 DLL that channel is about to change
   if ( Bt848PreChannelChange( channel ) != Success )
      return ( (LONG)TV_ERR_DLL_ERROR );

   // send the codes to the tuner via i2c
   LONG status = SetChannelAFT( channel, mode );
   if ( status != TV_ERR_SUCCESS )
      return ( status );

   // notify Bt848 DLL that channel has been changed
   if ( Bt848ChannelChanged( channel ) != Success )
      return ( (LONG)TV_ERR_DLL_ERROR );

   // store channel number
   m_channel = channel;

   return ( (LONG)TV_ERR_SUCCESS );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  int GetChannel( void )
// Purpose: Get channel
// Input:   None
// Output:  None
// Return:  Channel number
/////////////////////////////////////////////////////////////////////////////
int Tuner::GetChannel( void )
{
   return ( m_channel );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  WORD Tuner::MaxNumberOfChannel( int mode )
// Purpose: Return maximum number of valid channel
// Input:   int mode - air or cable
// Output:  None
// Return:  Maximum number of valid channel number
/////////////////////////////////////////////////////////////////////////////
WORD Tuner::MaxNumberOfChannel( int mode )
{
   if ( mode == modeTV )
      return ( sizeof(airNtscChannelCode) / sizeof(*airNtscChannelCode) - 1 );
   else
      return ( sizeof(cableNtscChannelCode) / sizeof(*cableNtscChannelCode) - 1 );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  WORD Tuner::GetDividerWord( int channel, int mode )
// Purpose: Obtain divider to change channel
// Input:   int channel - channel to obtain
//          int mode    - channel mode (air or cable)
// Output:  None
// Return:  Divider info if success, else 0
/////////////////////////////////////////////////////////////////////////////
WORD Tuner::GetDividerWord( int channel, int mode )
{
   // first figure out which TunerCode array
   // to use based on the video format & mode
   m_ptrTunerCodeArray	= NULL;
   if ( m_format == formatNTSC )
   {
      if ( mode == modeTV )
         m_ptrTunerCodeArray = airNtscChannelCode;
      else if ( mode == modeCable )
         m_ptrTunerCodeArray = cableNtscChannelCode;
   }
   else if ( m_format == formatPAL )
   {
      // add support for PAL later
      m_ptrTunerCodeArray = NULL;
   }

   // now look up the WORD in the selected array
   if ( m_ptrTunerCodeArray != NULL )
      return( m_ptrTunerCodeArray[ channel ] );
   else
      return( 0 );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void Tuner::SetDividerWord( int channel, int mode, WORD value )
// Purpose: Set divider info
// Input:   int channel - channel to obtain
//          int mode    - channel mode (air or cable)
//          WORD value  - value to set to
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void Tuner::SetDividerWord( int channel, int mode, WORD value )
{
   // first figure out which TunerCode array
   // to use based on the video format & mode
   m_ptrTunerCodeArray	= NULL;
   if ( m_format == formatNTSC )
   {
      if ( mode == modeTV )
         m_ptrTunerCodeArray = airNtscChannelCode;
      else if ( mode == modeCable )
         m_ptrTunerCodeArray = cableNtscChannelCode;
   }
   else if ( m_format == formatPAL )
   {
      // add support for PAL later
      m_ptrTunerCodeArray = NULL;
   }

   // now set the WORD in the selected array
   if ( m_ptrTunerCodeArray != NULL )
      m_ptrTunerCodeArray[ channel ] = value;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  WORD Tuner::GetControlWord( int channel, int mode)
// Purpose: Obtain control code to change band
// Input:   int channel - channel to obtain
//          int mode    - channel mode (air or cable)
// Output:  None
// Return:  Control info if success, else 0
/////////////////////////////////////////////////////////////////////////////
WORD Tuner::GetControlWord( int channel, int mode)
{
   // first figure out which TunerBandStruct array
   // to use based on the video format & mode
   m_ptrTunerCodeArray	= NULL;
   if ( m_format == formatNTSC )
   {
      if ( mode == modeTV )
         m_ptrTunerBandStructArray = airNtscBands;
      else if ( mode == modeCable )
         m_ptrTunerBandStructArray = cableNtscBands;
   }
   else if ( m_format == formatPAL )
   {
      // add support for PAL later
      m_ptrTunerBandStructArray = NULL;
   }

   // now figure out which band this channel is in
   int band = 0;
   for( int n = 0; m_ptrTunerBandStructArray[n].minChannel != 0; n++ )
   {
      if ( (channel >= m_ptrTunerBandStructArray[n].minChannel ) &&
           (channel <= m_ptrTunerBandStructArray[n].maxChannel ) )
         band = m_ptrTunerBandStructArray[n].band;
   }

   // now return the control word
   if ( m_ptrControlWordArray != NULL )
      return ( m_ptrControlWordArray[ band ] );
   else
      return( 0 );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  LONG Tuner::SetChannelAFT( int channel, int mode )
// Purpose: Set channel with Automatic Fine Turning
// Input:   int channel - channel to obtain
//          int mode    - channel mode (air or cable)
// Output:  None
// Return:  TV control error value
// Note:    This function was provided by Intel Corp. and has been modified
//          for our need.
/////////////////////////////////////////////////////////////////////////////
LONG Tuner::SetChannelAFT( int channel, int mode )
{

#define PLL_SETTLE_DELAY   30  // check each 30 millisec to see if PLL settled
                               // pll can take up to 150millisec to settle
#define ITERATIONS    20    // # iterations in AFT loop & in search table
#define PLLATTEMPTS   15    // # attempts to lock in PLL for each iteration
                            // PLLATTEMPTS * PLL_SETTLE_DELAY should be ~150msec

    DWORD time;             // get the current time
    BYTE  status;           // Tuner status, including AFC A/D value
    int   n;                // iteration count
    int   m=0;              // how many times was PLL not locked?
    BOOL  centered = FALSE; // "tuned in" flag
    int   freq;             // our temporary copy of frequency bits

//--------------------------
// Array of frequency offset values for AFT search
// This zig-zags -- -1/4 MHz, +1/4MHz, -1/2MHz, etc., to +2-1/2 MHz
//--------------------------
    int  freqdelta[ITERATIONS] =
       { -4,  4,  -8,  8, -12, 12, -16, 16,
        -20, 20, -24, 24, -28, 28, -32, 32,
        -36, 36, -40, 40 };

//--------------------------
// Notify Capture Driver to Stop Capture while we change channels,
//   mostly to prevent garbage data from going to VBI decoder.
//--------------------------
//    StopCapture();

//--------------------------
// Grab frequency from table (2 tables -- one for cable, one for antenna).
// Write initial values to frequency and control regs in tuner
//--------------------------
   WORD div = GetDividerWord( channel, mode );
   WORD con = GetControlWord( channel, mode );

//--------------------------
// Send frequency to I2C to set channel
//--------------------------
   freq = div;
   if ( Bt848I2CWrite( (BYTE)m_I2CAddress, div, con ) == Fail )
      return ( TV_ERR_I2C_ERROR );

//----------------------------------------------------
// Loop to settle in on final frequency
// First, delay to allow tuner PLL to settle
//   Tuner specs claim 150 msec as settling time
//   We are delaying a shorter time, then checking for lock.
//----------------------------------------------------
    for (n=0; (n < ITERATIONS) && (m < PLLATTEMPTS); ) {
        time = timeGetTime(); // get current time
        while (timeGetTime() - time < PLL_SETTLE_DELAY ) {
                // SPIN waiting for PLL_SETTLE_DELAY
        }

//--------------------------
// Now take a look at status, see if we are centered on a channel.
//   (Bit 6 should be high, indicating Phase Lock).
//   (Bits 2-0 should be value 2, indicating centered frequency).
//   (if bits 2-0 < 2, then we should raise frequency)
//   (if bits 2-0 > 2, then we should lower frequency)
//   (if bits 2-0 == 4, then we also may not be near anything!)
//--------------------------
         if ( I2CHWRead( (BYTE)m_I2CAddress, &status ) == Fail )
            return ( TV_ERR_I2C_ERROR );

//--------------------------
// Test Phase lock condition (bit 6)
//  (if phase lock is not achieved, we will just repeat the loop,
//   without changing frequency, giving PLL more time to settle out).
//--------------------------
         if ( status & 0x40 ) {

//--------------------------
// PLL Locked, test AFC condition (low 3 bits of status)
//--------------------------
            switch ( status & 0x07 )
            {
               case 0:     // carrier sensed, need to lower frequency
                  freq = freq - 2;
                  if ( I2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               case 1:     // carrier sensed, need to lower frequency
                  freq = freq - 1;
                  if ( I2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               case 2:     // carrier sensed, we are on correct frequency
                  centered = TRUE;
                  break;

               case 3:     // carrier sensed, need to raise frequency
                  freq = freq + 1;
                  if ( I2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               case 4:     // carrier not sensed, search using offset table
                  freq = div + freqdelta[n];
                  if ( I2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) ) == Fail )
                     return ( TV_ERR_I2C_ERROR );
                  break;

               default:
                  break;

            } // switch

//------------
// Increment n to control loop and access next frequency search offset
//------------
            n++;
            m = 0;         // Reset PLL lock attempt counter

        } // if phase locked

//--------------------------
// Else PLL not locked.
// Don't increment n.  Simply loop again until PLL is locked.
//--------------------------
        else {
            m++;
        } // else not phase locked

//--------------------------
// If we detected centered frequency,
//   update our table of frequencies (for fast tuning next time),
//   break out of loop (we are done),
// else repeat loop.
// If phase lock had not been detected, we did not change tuner settings,
//   we will just repeat the time delay, and check again for phase lock.
//--------------------------
        if (centered) {
            SetDividerWord( channel, mode, (WORD)freq );
            break; // out of for loop

        } // if centered

    } // for ITERATIONS iterations or for max PLLATTEMPTS to lock

//----------------------------------------------------
// Tuning loop is done, let's see how we did
//----------------------------------------------------

//--------------------------
// If we found a carrier, restart Capture.
// Notify Capture Driver of current channel
//--------------------------
   if ( centered )
   {
      return ( TV_ERR_SUCCESS );
   }

//--------------------------
// Else no carrier, return why we didn't tune right
//--------------------------
   else
   {
      // if we didn't find a carrier, restore I2C to table value
      I2CHWWrite3( (BYTE)m_I2CAddress, HIBYTE( freq ), LOBYTE( freq ) );
      return ( TV_ERR_NO_CARRIER );
   }
}


//===========================================================================
// Following are the functions communicate with I2C
//===========================================================================

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Bt848I2CInit( void )
// Purpose: Initialize I2C
// Input:   None
// Output:  None
// Return:  Success or Fail
// Note:    This function is provided for the purpose to communicate to I2C
/////////////////////////////////////////////////////////////////////////////
ErrorCode Bt848I2CInit( void )
{
   if ( I2CIsInitOK() == false )
      return ( Fail );

   I2CInitHWMode( 100000 );    // assume frequency = 100Khz
   return ( Success );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Bt848I2CWrite( BYTE I2CAddress, WORD msw, WORD lsw )
// Purpose: Write data to I2C
// Input:   BYTE I2CAddress - address of I2C
//          WORD msw        - most sigificant word
//          WORD lsw        - least sigificant word
// Output:  None
// Return:  Success or Fail
// Note:    This function is provided for the purpose to communicate with I2C
/////////////////////////////////////////////////////////////////////////////
ErrorCode Bt848I2CWrite( BYTE I2CAddress, WORD msw, WORD lsw )
{
   // write HIWORD to I2C
   if ( I2CHWWrite3( I2CAddress, HIBYTE( msw ), LOBYTE( msw ) ) == Fail )
      return ( Fail );

   // write LOWORD to I2C
   if ( I2CHWWrite3( I2CAddress, HIBYTE( lsw ), LOBYTE( lsw ) ) == Fail )
      return ( Fail );

   return ( Success );
}


//===========================================================================
// DLL Entry Point
//===========================================================================

/////////////////////////////////////////////////////////////////////////////
// Method:  (For Borland)   int PASCAL DllEntryPoint( HINSTANCE, DWORD dwReason, LPVOID )
//          (For Microsoft) int PASCAL DllMain( HINSTANCE, DWORD dwReason, LPVOID )
// Purpose: Specify DLL entry point and initialize I2C
// Input:   HINSTANCE      - no used
//          DWORD dwReason - the reason Dll is called
//          LPVOID         - no used
//
// Output:  None
// Return:  TRUE if success, else FALSE
/////////////////////////////////////////////////////////////////////////////
#ifdef __BORLANDC__
int PASCAL DllEntryPoint( HINSTANCE hInstance, DWORD dwReason, LPVOID )
#else  // Microsoft
int PASCAL DllMain( HINSTANCE hInstance, DWORD dwReason, LPVOID )
#endif
{
   // Initialize I2C when DLL is called
   if ( dwReason == DLL_PROCESS_ATTACH )
   {
      // Initialize I2C
      if ( Bt848I2CInit() == Fail )
         return ( FALSE );

      // Initialize GPIO
      if ( GPIOIsInitOK() == false )
         return ( FALSE );

      // set GPIO to normal mode
      SetGPIOMODE( GPIO_NORMAL );

      // start up audio with TV Tuner
      SetAudioSource( AUDIO_SOURCE_TVTUNER );

      // start up audio not mute
      SetAudioMute( 1 );

      hInst = hInstance;      // save handle to instance for later
   }
   else if ( dwReason == DLL_PROCESS_DETACH )
   {
      // mute audio when terminates
      SetAudioSource( AUDIO_SOURCE_MUTE );
   }

   return ( TRUE );
}


