
 /* NOTICE:                                             */
 /* THIS CODE IS NOT SUPPORTED, WARRANTED OR GUARANTEED */
 /* IN ANY WAY BY BROOKTREE CORPORATION.                */
 /* IT IS PROVIDED "AS IS" BECAUSE WE BELIEVE IT MAY    */
 /* BE USEFUL TO YOU.                                   */


/////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 1996 Brooktree Corporation
//
//  Module:
//
//    Tuner.h
//
//  Abstract:
//
//    Bt848 Tuner class header file
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __TUNER_H
#define __TUNER_H

#include "retcode.h"

/////////////////////////////////////////////////////////////////////////////
// Typedefs
/////////////////////////////////////////////////////////////////////////////
typedef WORD TunerCodes;
typedef TunerCodes* ptrTunerCodes;
typedef WORD ControlWord;


/////////////////////////////////////////////////////////////////////////////
// tuner brands
/////////////////////////////////////////////////////////////////////////////
enum { brandPhilips, brandTemic };


/////////////////////////////////////////////////////////////////////////////
// tuner has 2 different tuning modes: tv (ie. broadcast) or cable
/////////////////////////////////////////////////////////////////////////////
enum { modeTV, modeCable };


/////////////////////////////////////////////////////////////////////////////
// video formats
/////////////////////////////////////////////////////////////////////////////
enum { formatNTSC, formatPAL };


////////////////////////////////////////////////////////////////////
// tuner uses 3 different bands based on desired channel & mode
/////////////////////////////////////////////////////////////////////////////
enum { bandVHF_LOW, bandVHF_HIGH, bandUHF };


/////////////////////////////////////////////////////////////////////////////
// struct for associating a channel with a band
/////////////////////////////////////////////////////////////////////////////
typedef struct tagTunerBandStruct
{
   int minChannel;
   int maxChannel;
   int band;
} TunerBandStruct;
typedef TunerBandStruct* ptrTunerBandStruct;



/////////////////////////////////////////////////////////////////////////////
// Tuner Class
//
// Description:
//    This class encapsulates the tuner behavior of the Bt848. It provides
//    "SetChannel()" and "GetChannel()" to set and retrieve channel using the
//    tuner. To instantiate this class, the caller needs to supply 3 parameters
//    as defined in the constructor:
//       Tuner( int brand, int format, BYTE I2CAddress );
//          where brand can be either brandPhilips or brandTemic,
//                format can be either formatNTSC or formatPAL,
//                I2CAddress is the address of the tuner.
//    This class also incorporate Automatic Fine Tuning (AFT) when tuning a
//    channel. It will scan up to +/- 2 1/2 MHz from the predefined setting
//    in order to obtain optimum image of the channel.
//    
/////////////////////////////////////////////////////////////////////////////
class Tuner
{
   int  m_channel;      // which channel is set to
   int  m_format;       // video foramt (ntsc or pal)
   BYTE m_I2CAddress;   // I2C address

   // ptrs to lookup tables
   ptrTunerCodes	    m_ptrTunerCodeArray;
   ptrTunerBandStruct m_ptrTunerBandStructArray;
   ControlWord *      m_ptrControlWordArray;

   ////////////////////////////////////
   // Internal functions
   ////////////////////////////////////
   WORD      Tuner::GetDividerWord( int channel, int mode );
   void      Tuner::SetDividerWord( int channel, int mode, WORD value );
   WORD      Tuner::GetControlWord( int channel, int mode );
   LONG      Tuner::SetChannelAFT ( int channel, int mode );
   WORD      Tuner::MaxNumberOfChannel( int mode );

public:
   Tuner( int brand, int format, BYTE I2CAddress );
   ~Tuner();

   ////////////////////////////////////
   // Public member functions
   ////////////////////////////////////
   LONG      SetChannel( int channel, int mode );
   int       GetChannel( void );
};


#endif // __TUNER_H
